package com.v1.controller;

import com.alibaba.fastjson.JSONObject;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.entity.ApiMerchants;
import com.hai.entity.ApiOrderRecord;
import com.hai.entity.HighUser;
import com.hai.entity.OutRechargeOrder;
import com.hai.model.ResponseData;
import com.hai.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import jdk.jfr.consumer.RecordedObject;
import org.apache.commons.collections4.MapUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

/**
 * @serviceName rechargeProductController.java
 * @author Sum1Dream
 * @version 1.0.0
 * @Description // 充值产品接口
 * @createTime 17:14 2022/6/9
 **/
@Controller
@RequestMapping(value = "/rechargeProduct")
@Api(value = "充值产品接口")
public class RechargeProductController {

    private static Logger log = LoggerFactory.getLogger(GzSinopecController.class);

    @Resource
    private ApiMerchantsService apiMerchantsService;

    @Resource
    private SecConfigService secConfigService;

    @Resource
    private OutRechargeOrderService outRechargeOrderService;

    @Resource
    private ApiOrderRecordService apiOrderRecordService;

    @RequestMapping(value = "/getRechargeProduct", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "获取充值产品")
    public ResponseData getRechargeProduct(@RequestBody JSONObject object , HttpServletRequest request ) {
        try {

            String ip = "";
            // 有的user可能使用代理，为处理用户使用代理的情况，使用x-forwarded-for
            if  (request.getHeader("x-forwarded-for") == null)  {
                ip = request.getRemoteAddr();
            }  else  {
                ip = request.getHeader("x-forwarded-for");
            }

            if (!secConfigService.isConfig("IP_WHITE" , ip)) {
                log.error("getRechargeProduct error!", "非法ip地址，请联系管理人员！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "非法ip地址，请联系管理人员！");
            }

            if (object.getString("apiKey") == null ||
                    object.getString("merchId") == null ||
                    object.getString("sign") == null ||
                    object.getInteger("rechargeType") == null
            ) {
                log.error("getRechargeProduct error!", "请求参数校验失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR);
            }

            ApiMerchants apiMerchants = apiMerchantsService.findById(object.getLong("merchId"));

            if (apiMerchants == null) {
                log.error("getRechargeProduct error!", "商户号错误！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "商户号错误！");
            }

            Map<String , Object> map = new HashMap<>();
            map.put("apiKey" , object.getString("apiKey"));
            map.put("merchId" , object.getString("merchId"));
            map.put("rechargeType" , object.getString("type"));
            map.put("sign" , object.getString("sign"));

            if (!secConfigService.isSignVerify(map)) {
                log.error("getRechargeProduct error!", "签名校验失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.SIGN_VERIFY);
            }

            return ResponseMsgUtil.success(apiMerchantsService.getMerchProduct(apiMerchants.getId() , object.getInteger("rechargeType")));

        } catch (Exception e) {
            log.error("HighOpenApiController --> getRechargeProduct() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/checkAccountBalance", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "查询帐户余额")
    public ResponseData checkAccountBalance(@RequestBody JSONObject object , HttpServletRequest request ) {
        try {

            String ip = "";
            // 有的user可能使用代理，为处理用户使用代理的情况，使用x-forwarded-for
            if  (request.getHeader("x-forwarded-for") == null)  {
                ip = request.getRemoteAddr();
            }  else  {
                ip = request.getHeader("x-forwarded-for");
            }

            if (!secConfigService.isConfig("IP_WHITE" , ip)) {
                log.error("getRechargeProduct error!", "非法ip地址，请联系管理人员！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "非法ip地址，请联系管理人员！");
            }

            if (object.getString("apiKey") == null ||
                    object.getString("sign") == null ||
                    object.getString("merchId") == null
            ) {
                log.error("getRechargeProduct error!", "请求参数校验失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR);
            }

            ApiMerchants apiMerchants = apiMerchantsService.findById(object.getLong("merchId"));

            if (apiMerchants == null) {
                log.error("getRechargeProduct error!", "商户号错误！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "商户号错误！");
            }

            Map<String , Object> map = new HashMap<>();
            map.put("apiKey" , object.getString("apiKey"));
            map.put("merchId" , object.getString("merchId"));
            map.put("sign" , object.getString("sign"));

            if (!secConfigService.isSignVerify(map)) {
                log.error("getRechargeProduct error!", "签名校验失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.SIGN_VERIFY);
            }

            return ResponseMsgUtil.success(apiMerchants.getAmounts());

        } catch (Exception e) {
            log.error("HighOpenApiController --> getRechargeProduct() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/queryOrderResult", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "查询订单结果")
    public ResponseData queryOrderResult(@RequestBody JSONObject object , HttpServletRequest request ) {
        try {

            String ip = "";
            // 有的user可能使用代理，为处理用户使用代理的情况，使用x-forwarded-for
            if  (request.getHeader("x-forwarded-for") == null)  {
                ip = request.getRemoteAddr();
            }  else  {
                ip = request.getHeader("x-forwarded-for");
            }

            if (!secConfigService.isConfig("IP_WHITE" , ip)) {
                log.error("getRechargeProduct error!", "非法ip地址，请联系管理人员！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "非法ip地址，请联系管理人员！");
            }

            if (object.getString("apiKey") == null ||
                    object.getString("merchId") == null ||
                    object.getString("sign") == null ||
                    object.getString("sourceOrderNo") == null

            ) {
                log.error("getRechargeProduct error!", "请求参数校验失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR);
            }

            ApiMerchants apiMerchants = apiMerchantsService.findById(object.getLong("merchId"));

            if (apiMerchants == null) {
                log.error("getRechargeProduct error!", "商户号错误！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "商户号错误！");
            }

            Map<String , Object> map = new HashMap<>();
            map.put("apiKey" , object.getString("apiKey"));
            map.put("merchId" , object.getString("merchId"));
            map.put("sourceOrderNo" , object.getString("sourceOrderNo"));
            map.put("sign" , object.getString("sign"));

            if (!secConfigService.isSignVerify(map)) {
                log.error("getRechargeProduct error!", "签名校验失败！");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.SIGN_VERIFY);
            }


            ApiOrderRecord apiOrderRecord = apiOrderRecordService.queryOrderResult(map);

            OutRechargeOrder rechargeOrder = outRechargeOrderService.findByOrderNo(apiOrderRecord.getOrderNo());

            JSONObject returnObject = new JSONObject();
            returnObject.put("sourceOrderNo" , apiOrderRecord.getSourceOrderNo());
            returnObject.put("callbackStatus" , apiOrderRecord.getStatus());
            returnObject.put("goodsId" , apiOrderRecord.getGoodsId());
            returnObject.put("type" , rechargeOrder.getType());
            returnObject.put("operatorName" , rechargeOrder.getOperatorName());
            returnObject.put("rechargeType" , rechargeOrder.getRechargeType());
            returnObject.put("rechargeContent" , rechargeOrder.getRechargeContent());
            returnObject.put("rechargePrice" , rechargeOrder.getRechargePrice());
            returnObject.put("payRealPrice" , rechargeOrder.getPayRealPrice());
            returnObject.put("createTimed" , rechargeOrder.getCreateTimed());
            returnObject.put("payStatus" , rechargeOrder.getPayStatus());
            returnObject.put("rechargeStatus" , rechargeOrder.getRechargeStatus());


            return ResponseMsgUtil.success(returnObject);

        } catch (Exception e) {
            log.error("HighOpenApiController --> getRechargeProduct() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }
}
